program Testkzkh
  ! Filter in the kzkh domain prestack
  ! Loop over x
  ! If x< xshot  allow only positive dipping
  ! If x~xshot allow horizontal dipping
  ! If x> xshot allow negative dipping
  ! Input: Offset domain CIG (before filter)
  !      : axis 1=z, 2=x, 3=hx
  ! Output: Angle-domain CIG (after filter)
  !      : axis 1=z, 2=x, 3=hx 
  ! Output2: transformed domain (after filter)
  !      : axis 1=z, 2=kz, 3=kh 
  ! Read input arrays
  ! loop over x, extract 2D slices (z,h)
  ! 2D FFT, filtering, Inv 2D FFT

  use sep
  use omp_lib
  use fourier_modOMP
  use ifport
  use smooth3d_mod
  use kzkh_mod  

  implicit none

  logical(4) :: success      
  logical                   :: verb, kzkhout, zhout, Lop, Dop
  integer                   :: sstat
  integer                   :: it, nt, n4, npzero, iaxzero, iaxmax, ind, UD
  integer                   :: nz, nx, nh, nkz, nkh, shotnum
  integer                   :: iz, ix, ih, ikz, ikh, ixshot 
  real                      :: dz, dx, dh, dkz, dkh 
  real                      :: oz, ox, oh, okz, okh 
  real                      :: Gx
  real :: pi=4.*atan(1.), val, rad, x, h, val1, val2, off, khmax, kzmax
  real, dimension(:,:,:), allocatable  :: model, model2, elsemodel2, model3, model4
  real, dimension(:,:), allocatable  :: weightArr, CompweightArr
  complex, dimension(:,:), allocatable  :: tmpArrZH1, tmpArrZH2, tmpArrKZKH1, tmpArrKZKH2
  complex, allocatable, dimension(:)  :: in1, in2, out1, out2

  character(len=128)        ::  out_, out2_, trans_, trans2_

  ! OMP stuff 
  integer :: inode,node,nodes,Totalnodes


  call sep_init()


  !!======================================================
  !! Parameter reading
  
  write(0,*) ' This is the beginning of kz-kh filter'

   call from_param("verb",verb,.false.);      
   call from_param("Lop",Lop,.false.);   ! [0] = down [1]=up
   call from_param("Dop",Dop,.false.);   ! [0] = down [1]=up
   call from_param('shotnum',shotnum,1);
   call from_param('UD',UD,1);           ! [1] = up-going  [2]=down-going


  if (shotnum .eq.  1) Gx= 52550.
  if (shotnum .eq.  2) Gx= 52950.
  if (shotnum .eq.  3) Gx= 53350.
  if (shotnum .eq.  4) Gx= 53750.
  if (shotnum .eq.  5) Gx= 54150.
  if (shotnum .eq.  6) Gx= 54550.
  if (shotnum .eq.  7) Gx= 54950.
  if (shotnum .eq.  8) Gx= 55350.
  if (shotnum .eq.  9) Gx= 55760.
  if (shotnum .eq. 10) Gx= 56150.
  if (shotnum .eq. 11) Gx= 56550.
  if (shotnum .eq. 12) Gx= 56950.
  if (shotnum .eq. 13) Gx= 57350.
  if (shotnum .eq. 14) Gx= 57750.
  if (shotnum .eq. 15) Gx= 58150.
  if (shotnum .eq. 16) Gx= 58550.

!55250-55750=-500
!53700-54150 = 450
!54650-54950=-300
!1025-300 = 725

  !! input (angle domain CIG)
   call from_history("n1", nz);
   call from_history("d1", dz);  call from_history("o1", oz);
   call from_history("n2", nx);
   call from_history("d2", dx);      call from_history("o2", ox);;
   call from_history("n3", nh);
   call from_history("d3", dh);      call from_history("o3", oh);
 
    nkz = nz
    okz = -1.*pi/(dz)
    dkz = 2.*pi/(2*nz*dz)

    nkh = nh
    okh = -1.*pi/(dh)
    dkh = 2.*pi/(2*nh*dh)

   !! Auxillary output files
   call from_param("outtag", out_);
   call from_param("elseouttag", out2_);
   call from_param("kzkhout",kzkhout,.false.);
   call from_param("zhout",zhout,.false.);
   if (kzkhout) then
	call from_param("kzkhfile", trans_);
	call from_param("kzkhfile2", trans2_);
   end if
   
   write(0,*) 'kzkhfile name', trans_
   write(0,*) 'trans2', trans2_
   write(0,*) 'kzkhout', kzkhout
!   call erexit('exit now')

   ! Parallel Programming
   call from_param("nodes" ,nodes, 16)
   !nodes=8
   Totalnodes =omp_get_num_procs()
   call omp_set_num_threads(nodes); 
   write(0,*) 'OMP: using', nodes , 'out of', Totalnodes
    

   !*********************************
   write(0,*) '**************************'
   write(0,*) ' nz, dz, oz ', nz, dz, oz
   write(0,*) ' nx, dx, ox',  nx, dx, ox
   write(0,*) ' nh, dh, oh',  nh, dh, oh
   write(0,*) ' nkz, dkz, okz', nkz, dkz, okz
   write(0,*) ' nkh, dkh, okh', nkh, dkh, okh
   write(0,*) 'zhout, kzkhout', zhout, kzkhout
   write(0,*) '**************************'

   !----------------------------------------------------------------

  !! output file (angle domain CIG)
   if (zhout) then
   call auxputch("n1",'i', nz, trim(out_));      call auxputch("label1",'s', 'z', trim(out_)); 
   call auxputch("d1",'f', dz, trim(out_));      call auxputch("o1",'f', oz, trim(out_));
   call auxputch("n2",'i', nx, trim(out_));  call auxputch("label2",'s', 'x', trim(out_)); 
   call auxputch("d2",'f', dx, trim(out_));      call auxputch("o2",'f', ox, trim(out_));
   call auxputch("n3",'i', nh, trim(out_));  call auxputch("label3",'s', 'h', trim(out_)); 
   call auxputch("d3",'f', dh, trim(out_));      call auxputch("o3",'f', oh, trim(out_));

   call auxputch("n1",'i', nz, trim(out2_));      call auxputch("label1",'s', 'z', trim(out2_)); 
   call auxputch("d1",'f', dz, trim(out2_));      call auxputch("o1",'f', oz, trim(out2_));
   call auxputch("n2",'i', nx, trim(out2_));  call auxputch("label2",'s', 'x', trim(out2_)); 
   call auxputch("d2",'f', dx, trim(out2_));      call auxputch("o2",'f', ox, trim(out2_));
   call auxputch("n3",'i', nh, trim(out2_));  call auxputch("label3",'s', 'h', trim(out2_)); 
   call auxputch("d3",'f', dh, trim(out2_));      call auxputch("o3",'f', oh, trim(out2_));

   end if


  if (kzkhout) then
  !! output tau-p transformed file

   write(0,*) 'writing kzkhout files', trim(trans_), trim(trans2_)
   call auxputch("n1",'i', 2*nkz, trim(trans_));      call auxputch("label1",'s', 'kz', trim(trans_)); 
   call auxputch("d1",'f', dkz, trim(trans_));      call auxputch("o1",'f', okz, trim(trans_));
   call auxputch("n2",'i', nx, trim(trans_));  call auxputch("label2",'s', 'x', trim(trans_)); 
   call auxputch("d2",'f', dx, trim(trans_));      call auxputch("o2",'f', ox, trim(trans_));
   call auxputch("n3",'i', 2*nkh, trim(trans_));  call auxputch("label3",'s', 'kh', trim(trans_)); 
   call auxputch("d3",'f', dkh, trim(trans_));      call auxputch("o3",'f', okh, trim(trans_));

   call auxputch("n1",'i', 2*nkz, trim(trans2_));      call auxputch("label1",'s', 'kz', trim(trans2_)); 
   call auxputch("d1",'f', dkz, trim(trans2_));      call auxputch("o1",'f', okz, trim(trans2_));
   call auxputch("n2",'i', nx, trim(trans2_));  call auxputch("label2",'s', 'x', trim(trans2_)); 
   call auxputch("d2",'f', dx, trim(trans2_));      call auxputch("o2",'f', ox, trim(trans2_));
   call auxputch("n3",'i', 2*nkh, trim(trans2_));  call auxputch("label3",'s', 'kh', trim(trans2_)); 
   call auxputch("d3",'f', dkh, trim(trans2_));      call auxputch("o3",'f', okh, trim(trans2_));

   end if

   !----------------------------------------------------------------
   ! Allocation

   write(0,*) 'allocating size', nz,nx,nh,nz*(nx*nh)
   allocate(model(nz,nx,nh),model2(nz,nx,nh))
   if (zhout) allocate(elsemodel2(nz,nx,nh))
   write(0,*) 'pass 2'
   if (kzkhout)   allocate(model3(2*nkz,nx,2*nkh), model4(2*nkz,nx,2*nkh))
   write(0,*) 'pass 3'
   allocate(tmpArrZH1(2*nz,2*nh),tmpArrZH2(2*nz,2*nh),tmpArrKZKH1(2*nkz,2*nkh),tmpArrKZKH2(2*nkz,2*nkh))
   write(0,*) 'pass 4'
   allocate(in1(2*nz),out1(2*nz),in2(2*nh),out2(2*nh))
   allocate(weightArr(2*nkz,2*nkh),CompweightArr(2*nkz,2*nkh))
!----------------------------------------------------------------
   ! initialize
  call ft_init(2*nz, 2*nh,in1,out1,in2,out2)
  
  write(0,*) 'pass 1', UD

  if (UD .eq. 1) then
     call initiateDipAng_Up()
  else
     call initiateDipAng_Down()
  end if
 


  write(0,*) 'pass 2'

      do ih=1,nh
      do ix=1,nx
      do iz=1,nz
         model(iz,ix,ih) = 0.;
         model2(iz,ix,ih) = 0.;
         if (zhout) elsemodel2(iz,ix,ih) = 0.; 
      end do
      end do
      end do

  write(0,*) 'pass 3'

   if (kzkhout) then
       do ikh=1,2*nkh
       do ix=1,nx
       do ikz=1,2*nkz
         model3(ikz,ix,ikh) = 0.;
         model4(ikz,ix,ikh) = 0.;
       end do
       end do
       end do
   end if

  write(0,*) 'pass 4'

   ! Read in ODCIG
   call sep_read(model)
write(0,*) 'after reading model', sum(model)

  write(0,*) 'pass 5'

   ! loop over x slice
   do ix=1,nx  
       x = (ix-1)*dx+ox
       
!       write(0,*) 'ix, x', ix, x

         do ih = 1,nh
           do iz = 1,nz
                 val =model(iz,ix,ih)
		 tmpArrZH1(iz+nz/2,ih) = val
           end do
         end do

	if (mod(ix,20) .eq. 0)  write(0,*) 'loop 1', ix

       !! 2D FFT in (x,h) axis  
       in1=0.; out1=0.;
       in2=0.; out2=0.;

       call ft1axis(.false.,-1.,tmpArrZH1,in1,out1)
       call ft2axis(.false.,-1.,tmpArrZH1,in2,out2)     

       !! Filtering in kz-kh
       do ikh=1,2*nkh
       do ikz=1,2*nkz
         tmpArrKZKH1(ikz,ikh) = 0.
         tmpArrKZKH2(ikz,ikh) = 0.
       end do
       end do

!  write(0,*) 'loop 2', ix, x, Gx

       call weightcone(weightArr,2*nkz,2*nkh,dkz,dkh,okz,okh,shotnum,x,Gx)

!  write(0,*) 'loop 3', ix

       call triangle2( 2, 2, 2*nkz, 2*nkh, weightArr, weightArr)
       do ikh=1,2*nkh
          do ikz=1,2*nkz
             CompweightArr(ikz,ikh) = 1.0 -weightArr(ikz,ikh)
          end do
       end do

!  write(0,*) 'loop 4', ix

       call EdgeTaper(weightArr,2*nkz,2*nkh,20,5)
       call EdgeTaper(CompweightArr,2*nkz,2*nkh,20,5)

!  write(0,*) 'loop 5', ix

   if (2 > 5) then
     call sep_write(weightArr, "weightArr.H");
     call auxputch("n1",'i', 2*nkz, "weightArr.H");      call auxputch("label1",'s', 'kz', "weightArr.H"); 
     call auxputch("d1",'f', dkz, "weightArr.H");      call auxputch("o1",'f', okz, "weightArr.H");
     call auxputch("n2",'i', 2*nkh, "weightArr.H");  call auxputch("label2",'s', 'kh', "weightArr.H"); 
     call auxputch("d2",'f', dkh, "weightArr.H");      call auxputch("o2",'f', okh, "weightArr.H");
     call erexit('exxit noww')
   end if

!  write(0,*) 'loop 6', ix


       do ikh=1,2*nkh
       do ikz=1,2*nkz
         tmpArrKZKH1(ikz,ikh) = tmpArrZH1(ikz,ikh)*(weightArr(ikz,ikh))
         tmpArrKZKH2(ikz,ikh) = tmpArrZH1(ikz,ikh)*(CompweightArr(ikz,ikh))
       end do
       end do

!  write(0,*) 'loop 7', ix
 
	if (kzkhout) then
   	 ! output the tau-p array
          do ikh = 1,2*nkh
            do ikz=1,2*nkz   
              val1 = abs(tmpArrKZKH1(ikz,ikh))
              val2 = abs(tmpArrKZKH2(ikz,ikh))
              model3(ikz,ix,ikh) = val1
              model4(ikz,ix,ikh) = val2
            end do
          end do
        end if

!  write(0,*) 'loop 8', ix

      !! Inverse 2D FFT transform (kz,kh) ---> (x,h)
       call ft2axis(.true.,-1.,tmpArrKZKH1,in2,out2)
       call ft1axis(.true.,-1.,tmpArrKZKH1,in1,out1)
       call ft2axis(.true.,-1.,tmpArrKZKH2,in2,out2)
       call ft1axis(.true.,-1.,tmpArrKZKH2,in1,out1)

       if (zhout) then
	! output the post-filtered array FFT in z
           do ih = 1,nh
              do iz=1,nz   
		 val1 = real(tmpArrKZKH1(iz+nz/2,ih))
		 val2 = real(tmpArrKZKH2(iz+nz/2,ih))
                 model2(iz,ix,ih) = val1
                 elsemodel2(iz,ix,ih) = val2
              end do
           end do
       end if

     end do


  
      ! Output result
	write(0,*) 'Output model', zhout, kzkhout
     if (zhout) then
      call sep_write(model2,out_,esize=4)   ! ADCIG
      call sep_write(elsemodel2,out2_,esize=4)   ! ADCIG
      end if
      if (kzkhout) then
         call sep_write(model3,trans_,esize=4)   
         call sep_write(model4,trans2_,esize=4)   
      end if

      call sep_close()
    
	write(0,*) 'dealloc model1'
	deallocate(model)
	write(0,*) 'dealloc model2'
	deallocate(model2)
        if (kzkhout) then
  	 write(0,*) 'dealloc model3'
 	 deallocate(model3)
	 write(0,*) 'dealloc model4'
	 deallocate(model4)
       end if

  write(0,*) 'pass 13'

 end program
 
